

/* Extensions */


// Extensions are used to add new functionality to an existing type, but they
// cannot override existing functionality.


// 1. Computed Properties

extension Int {
   var square:Int { return self * self }
}

4.square


// Useful example from book:
extension Double {
   var m: Double { return self }
   var cm: Double { return self / 100.0 }
   var ft: Double { return self / 3.28084 }
}

1.m

1.cm

3.cm + 2.m + 45.ft

// Note: No non-computed properties






// 2. Methods
extension Int {
   func repeet(task: ()->()) {
      for _ in 0..<self {
         task()
      }
   }
}

3.repeet(){
   print("hello")
}

extension Double {
   mutating func square() {
      self = self * self
   }
}

var d = 2.0  // cannot use let
d.square()
d

//2.0.square()   // Error: 2.0 cannot be modified. It's a literal.







// 3. Subscripts
extension String {
   subscript(var i: Int) -> Character {
      let index = self.startIndex.advancedBy(i)
      return self[index]
   }
}

let str = "Hello world"
str[4]





















// 4. Convenience Initializer

struct Templature {
   var fahrenheit: Double = 0
}

let t = Templature(fahrenheit: 75)


extension Templature {
   init(celsius: Double) {
      fahrenheit = celsius * 9 / 5 + 32
   }
}

// By using extension you can call default initilizer or memberwise initializer 
// in your own initializer.

//extension Templature {
//   init(celsius: Double) {
//      let x = celsius * 9 / 5 + 32
//      self.init(fahrenheit: x)
//   }
//}

// Note: No designated initializer









// 5. Nested Types
class Animal {
   enum Habitat {
      case GrassLand, Desert, Forest
   }
   var habitat:Habitat = .GrassLand
}
Animal.Habitat.Desert.hashValue

extension Animal {
   struct Attributes {
      var height:Double
      var age:Int
   }
   var attrib:Attributes { return Attributes(height: 0, age: 0)}
}

var x:Animal.Attributes


// 6. Protocol
protocol Printable {
   func printf() -> String
}

extension Animal : Printable {
   func printf()->String {
      return "I am an Animal"
   }
}



// Six types of extensions
// 1. Computed properties
// 2. Methods
// 3. Subscripts
// 4. Convenience initializers
// 5. Nested types
// 6. Protocols














